import sys
from StringIO import StringIO

from agenda import Agenda

class Node:
    def __init__(self, title, state, tags, level, priority, states):
        self._title = title
        self._state = state
        self._priority = priority
        self._tags = tags
        self._level = level
        
        self._states = states
        self._todo = []
        self._done = []
        for tododone in states:
            self._todo.extend(tododone[0])
            self._done.extend(tododone[1])
        
        self._ts = {
            'inactive': [],
            'active': [],
            'scheduled': [],
            'deadline': [],
            }
        
        self._body = []
        self._children = []
    
    def body(self, content = None):
        if content is None:
            return self._body
        else:
            self._body = content
    
    def append(self, node):
        self._children.append(node)
    
    def title(self):
        return self._title
    
    def tags(self):
        return self._tags
    
    def level(self):
        return self._level
    
    def state(self):
        return self._state
    
    def children(self):
        return self._children
    
    def dump(self, f):
        heading = self.heading()
        if heading:
            print >>f, heading.encode('utf-8')
        
        for l in self._body:
            print >>f, l.encode('utf-8'),
        
        for n in self._children:
            n.dump(f)
    
    def __str__(self):
        f = StringIO()
        self.dump(f)
        content = f.getvalue()
        f.close()
        return content
    
    def stars(self):
        l = ''
        for i in xrange(self._level):
            l+="*"
        return l
        
    def heading(self, stars = True):
        if self._title is not None:
            if stars:
                heading = self.stars() + " "
            else:
                heading = ""
            
            if self._state is not None:
                heading += str(self._state) + " "
            
            if self._priority is not None:
                heading += "[#" + str(self._priority) + "] "
            
            heading += self._title
            
            if len(self._tags) > 0:
                tags = ":" + ":".join(self._tags) + ":"
                if len(heading) < 77 - len(tags) - 1:
                    heading = heading.ljust(77 - len(tags))
                    heading += tags
                else:
                    heading += " " + tags
        else:
            heading = ""
        
        return heading
    
    def is_done(self):
        return self._state in self._done
    
    def is_todo(self):
        return self._state in self._todo
    
    def scheduled(self):
        dates = map(lambda x: x.begin(time=False), self._ts['scheduled'])
        if len(dates) > 0:
            dates.sort()
            return dates[0]
        else:
            return None
    
    def deadline(self):
        dates = map(lambda x: x.begin(time=False), self._ts['deadline'])
        if len(dates) > 0:
            dates.sort()
            return dates[0]
        else:
            return None
    
    def priority(self):
        return self._priority

